@@ -1,7 +1,7 @@
 package Convert::UU;
 
 use strict;
-use 5.003;
+BEGIN {require 5.004} # m//gc
 use vars qw($VERSION @ISA @EXPORT_OK);
 use Carp 'croak';
 
@@ -14,10 +14,7 @@ require Exporter;
 @EXPORT_OK = qw(
 	     uudecode uuencode
 );
-$VERSION = '0.04';
-
-
-# Preloaded methods go here.
+$VERSION = '0.05';
 
 #
 #  From comp.lang.perl 3/1/95.
@@ -32,25 +29,25 @@ sub uuencode {
     $mode ||= "644";
     $file ||= "uuencode.uu";
 
-    my($chunk,$result,$r);
-    $result = "";
+    my($chunk,@result,$r);
     if (
+	ref($in) eq 'IO::Handle' or
 	ref(\$in) eq "GLOB" or 
-	ref($in) eq 'GLOB' or 
-	ref($in) eq 'FileHandle' or
-	ref($in) eq 'IO::Handle'
+	ref($in) eq "GLOB" or 
+	ref($in) eq 'FileHandle'
        ) {
-	local $^W = 0; # Why did I get use of undefined value here ?
+        # local $^W = 0; # Why did I get use of undefined value here ?
+	binmode($in);
 	while (defined($r = read($in,$chunk,45)) && $r > 0) {
-	    $result .= uuencode_chunk($chunk);
+	    push @result, uuencode_chunk($chunk);
 	}
     } elsif (ref(\$in) eq "SCALAR") {
-	while ($in =~ s/(.{1,45})//s) {
-	    $result .= uuencode_chunk($1);
+        pos($in)=0;
+	while ($in =~ m/\G(.{1,45})/sgc) {
+	    push @result, uuencode_chunk($1);
 	}
     }
-    return "begin $mode $file\n" . $result . "end\n";
-
+    join "", "begin $mode $file\n", @result, "end\n";
 }
 
 sub uuencode_chunk {
@@ -64,7 +61,7 @@ sub uuencode_chunk {
 #    chop($encoded_string);                          # remove trailing \n
 #    $encoded_string =~ tr#`!-_#A-Za-z0-9+/#;        # tr to mime alphabet
 #    substr($encoded_string, $mod3) =~ tr/A/=/;      # adjust padding
-    return $encoded_string;
+    $encoded_string;
 }
 
 sub uudecode {
@@ -72,25 +69,26 @@ sub uudecode {
       unless(@_ == 1);
     my($in) = @_;
 
-    my($result,$file,$mode);
-    $result = $mode = $file = "";
+    my(@result,$file,$mode);
+    $mode = $file = "";
     if (
+	ref($in) eq 'IO::Handle' or
 	ref(\$in) eq "GLOB" or
-	ref($in) eq 'GLOB'
-	or ref($in) eq 'FileHandle'
-	or ref($in) eq 'IO::Handle'
+	ref($in) eq "GLOB" or
+	ref($in) eq 'FileHandle'
        ) {
 	local($\) = "\n";
+	binmode($in);
 	while (<$in>) {
 	    if ($file eq "" and !$mode){
 		($mode,$file) = /^begin\s+(\d+)\s+(\S+)/ ;
 		next;
 	    }
 	    last if /^end/;
-	    $result .= uudecode_chunk($_);
+	    push @result, uudecode_chunk($_);
 	}
     } elsif (ref(\$in) eq "SCALAR") {
-	while ($in =~ s/(.*?\n)//s) {
+	while ($in =~ m/\G(.*?\n)/gc) {
 	    my $line = $1;
 	    if ($file eq "" and !$mode){
 		($mode,$file) = $line =~ /^begin\s+(\d+)\s+(\S+)/ ;
@@ -98,7 +96,7 @@ sub uudecode {
 	    }
 	    next if $file eq "" and !$mode;
 	    last if $line =~ /^end/;
-	    $result .= uudecode_chunk($line);
+	    push @result, uudecode_chunk($line);
 	}
     } elsif (ref($in) eq "ARRAY") {
 	my $line;
@@ -109,10 +107,10 @@ sub uudecode {
 	    }
 	    next if $file eq "" and !$mode;
 	    last if $line =~ /^end/;
-	    $result .= uudecode_chunk($line);
+	    push @result, uudecode_chunk($line);
 	}
     }
-    wantarray ? ($result,$file,$mode) : $result;
+    wantarray ? (join("",@result),$file,$mode) : join("",@result);
 }
 
 sub uudecode_chunk {
@@ -123,11 +121,11 @@ sub uudecode_chunk {
 #    my $return = unpack("u", $string);
 #    warn "DEBUG: return [$return]";
 #    $return;
-    local $^W = 0; # Bug in perl5.002 ? I get a Use of unini.. with ANY $string
-    return unpack("u", $string);
+    unpack("u", $string);
 }
 
 1;
+
 __END__
 
 =head1 NAME
@@ -162,15 +160,11 @@ string, the second is the filename and the third is the mode.
 
 Both uudecode and uuencode are in @EXPORT_OK.
 
-=head1 PORTABILITY
-
-No effort has been made yet to port this module to non UNIX operating
-systems. Volunteers are welcome.
-
 =head1 AUTHOR
 
-Andreas Koenig E<lt>andreas.koenig@mind.deE<gt>. With code stolen from
-Hans Mulder E<lt>hansm@wsinti05.win.tue.nlE<gt> and Randal L. Schwartz
+Andreas Koenig E<lt>andreas.koenig@mind.deE<gt>. With code integrated
+that was posted to USENET from Hans Mulder
+E<lt>hansm@wsinti05.win.tue.nlE<gt> and Randal L. Schwartz
 E<lt>merlyn@teleport.comE<gt>.
 
 =head1 SEE ALSO
